#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 17-7-8
# @Author  : JCai
# @desc    :

import os
import django

from info_sync.views.listing_sync_info import info_variation_change_to_sql, info_base_change_to_sql, \
    info_more_change_to_sql
from utils.util import str_to_date

os.environ['DJANGO_SETTINGS_MODULE'] = 'EBayClub.settings'
django.setup()

from datetime import datetime, timedelta

import pytz

from info_sync.interface.trading_inter import MyEbay
from info_sync.models.listing_info_models import EBayListing, EBayListingInfoMore, EBayListingVariations
from utils.logger import logStatusBegin, logStatusEnd, logInfo
from info_sync.models.account_models import EBayAccount
from info_sync.tasks import listing_more_info_sync_task


def listing_info_change_sync(store_obj, from_time, to_time, item_list):
    logStatusBegin('Listing Info Change Sync Begin')
    day_now = datetime.now(pytz.UTC) + timedelta(minutes=2)
    mod_time_from = str(day_now - timedelta(hours=26)).split('.')[0].replace(' ', 'T') + 'Z'
    mod_time_to = str(day_now).split('.')[0].replace(' ', 'T') + 'Z'
    call_data = {
        'EndTimeFrom': from_time,
        'EndTimeTo': to_time,
        'ModTimeFrom': mod_time_from,
        'ModTimeTo': mod_time_to,
        'IncludeWatchCount': True,
        'OutputSelector': ['ItemArray.Item.ItemID',
                           'ItemArray.Item.SKU',
                           'ItemArray.Item.SellingStatus.ListingStatus',
                           ],
        'DetailLevel': 'ReturnAll',
    }
    token = store_obj.get('Token', '')
    item_array = []
    if not token or len(token) <= 0:
        logInfo('Sync ERROR:\t%s\n\t%s' % (store_obj.get('name', ''), 'Token is Empty!'))
        logStatusEnd('Listing Info Change Sync END')
        return item_array

    my_ebay = MyEbay({'Token': str(token), 'CertID': '', 'DevID': '', 'AppID': ''})
    try:
        status, message, listing_items = my_ebay.get_seller_events(call_data)
        if not status:
            logInfo('Sync Info: %s\n\t%s' % (store_obj.get('name', ''), message))
            logStatusEnd('Listing Info Change Sync END')
            return item_array

        if type(listing_items) == type({}):
            listing_items = [listing_items]

        logInfo('Sync Info: Update Number\t%s' % str(len(listing_items)))
        for each in listing_items:
            try:
                item_id = each.get('ItemID', '')
                listing_status = each.get('SellingStatus', {}).get('ListingStatus', '')
                if item_id not in item_list:
                    if listing_status == 'Active':
                        item_array.append(item_id)
            except Exception, e:
                logInfo('SQL ERROR:\t%s\n\t%s' % (store_obj.get('name', ''), str(e)))
    except Exception, e:
        logInfo('Sync ERROR:\t%s\n\t%s' % (store_obj.get('name', ''), str(e)))

    logStatusEnd('Listing Info Change Sync END')
    return item_array


def listing_base_selector_keys():
    return ['ItemArray.Item.ItemID',
            'ItemArray.Item.Site',
            'ItemArray.Item.SellingStatus.ListingStatus',
            'ItemArray.Item.ListingType',
            'ItemArray.Item.SKU',
            'ItemArray.Item.Title',
            'ItemArray.Item.SubTitle',
            'ItemArray.Item.Currency',
            'ItemArray.Item.SellingStatus.CurrentPrice',
            'ItemArray.Item.StartPrice',
            'ItemArray.Item.ReservePrice',
            'ItemArray.Item.BuyItNowPrice',
            'ItemArray.Item.ListingDuration',
            'ItemArray.Item.Quantity',
            'ItemArray.Item.SellingStatus.QuantitySold',
            'ItemArray.Item.ListingDetails.StartTime',
            'ItemArray.Item.ListingDetails.EndTime',
            'ItemArray.Item.Variations',
            'ItemArray.Item.Country',
            'ItemArray.Item.Location',
            'PaginationResult']


def listing_base_info_sync(store_obj, to_time, from_time):
    logStatusBegin('Listing Base Info Sync Begin')
    item_array = []
    store_name = store_obj.get('name', '')
    token = store_obj.get('Token', '')
    if not token or len(token) <= 0:
        logInfo('Sync error:\t%s\n\t%s' % (store_name, 'Token is Empty!'))
        logStatusEnd('Listing Base Info Sync END')
        return item_array

    store_id = store_obj.get('account_id', '')
    if not store_id or store_id <= 0:
        logInfo('sync error:\t%s\n\t%s' % (store_name, 'Store ID is Empty!'))
        logStatusEnd('Listing Base Info Sync END')
        return

    my_ebay = MyEbay({'Token': str(token), 'CertID': '', 'DevID': '', 'AppID': ''})
    call_data = {
        'EndTimeFrom': from_time,
        'IncludeWatchCount': True,
        'EndTimeTo': to_time,
        'IncludeVariations': True,
        'DetailLevel': 'ReturnAll',
        'OutputSelector': listing_base_selector_keys(),
        'Pagination': {'EntriesPerPage': 200},
        'GranularityLevel': 'Fine'
    }
    page = 1
    while True:
        try:
            logInfo('Sync Page: %s' % str(page))
            call_data['Pagination']['PageNumber'] = page
            status, message, listing_items, is_next_get = my_ebay.get_seller_list(call_data)
            if not status:
                logInfo('Sync Page: %s\t%s' % (store_name, message))
                break
            page += 1
            if type(listing_items) == type({}):
                listing_items = [listing_items]

            logInfo('Sync Page: Update Number\t%s' % str(len(listing_items)))
            for each in listing_items:
                item_id = each.get('ItemID', '')
                open_date = str_to_date(each.get('ListingDetails', {}).get('StartTime', ''))
                variations = each.get('Variations', {}).get('Variation', [])
                if type(variations) == type({}):  # if only have a variations  ,the resutl is a dict
                    variations = [variations]
                variation_list, variation_count = info_variation_change_to_sql(item_id, variations, open_date)
                info_base_change_to_sql(item_id, each, store_id, variation_count, open_date)

            if not is_next_get:
                logInfo('Sync Page: %s\t%s' % (store_name, message))
                break
        except Exception, e:
            logInfo('Sync ERROR:\t%s\n\t%s' % (store_name, str(e)))

    logStatusEnd('Listing Base Info Sync END')
    return item_array


def listing_more_selector_keys():
    return ['Item.ItemID',
            'Item.PrimaryCategory',
            'Item.SecondaryCategory',
            'Item.Storefront',
            # 'Item.VariationSpecificName'
            'Item.ConditionID',
            'Item.Description',
            'Item.PictureDetails.PictureURL',
            'Item.DispatchTimeMax',
            'Item.ShippingDetails.ShippingServiceOptions',
            'Item.ShippingDetails.InternationalShippingServiceOption',
            'Item.ShippingDetails.ExcludeShipToLocation',
            'Item.ReturnPolicy',
            'Item.BuyerRequirementDetails',
            'Item.ItemSpecifics.NameValueList',
            'Item.Variations',
            'Item.PaymentMethods',
            'Item.PayPalEmailAddress',
            'Item.PaymentInstructions',
            'Item.PostalCode']


def listing_more_info_sync(item_ids, store_obj):
    logStatusBegin('Listing More Info Sync Begin')
    token = store_obj.get('Token', '')
    if not token or len(token) <= 0:
        logInfo('sync error:\t%s\n\t%s' % (store_obj.get('name', ''), 'Token is Empty!'))
        logStatusEnd('Listing Base Info Sync END')
        return

    call_data = {
        'IncludeVariations': True,
        'IncludeItemSpecifics': True,
        'GranularityLevel': 'Fine',
        'DetailLevel': 'ReturnAll',
        'OutputSelector': listing_more_selector_keys(),
    }
    my_ebay = MyEbay({'Token': str(token), 'CertID': '', 'DevID': '', 'AppID': ''})
    count = len(item_ids)
    number = 1
    for item_id in item_ids:
        logInfo('%5d/%5d\tItemID:%s' % (number, count, item_id))
        try:
            status, message, listing_items = my_ebay.get_item(call_data)
            number += 1
            if not status:
                logInfo('sync info: %s\n%s' % (store_obj.get('name', ''), message))
                continue
            try:
                info_more_change_to_sql(item_id, listing_items)
            except Exception, e:
                logInfo('SQL error:\t%s\n\t%s' % (store_obj.get('name', ''), str(e)))

        except Exception, e:
            logInfo('sync error:\t%s\n\t%s' % (store_obj.get('name', ''), str(e)))
    logStatusEnd('Listing More Info Sync END')


def ebay_listing_sync_daily(store):
    logStatusBegin('Listing Sync Begin:%s' % store.get('name', ''))
    from_time = str(datetime.now(pytz.UTC) - timedelta(days=3)).split('.')[0].replace(' ', 'T') + 'Z'
    to_time = str(datetime.now(pytz.UTC) + timedelta(days=33)).split('.')[0].replace(' ', 'T') + 'Z'
    # 获取新增的Listing的SKU
    item_array = listing_base_info_sync(store, to_time, from_time)
    # 获取变化的Listing的SKU
    change_item = listing_info_change_sync(store, from_time, to_time, item_array)
    # 更新Listing更多的信息
    listing_more_info_sync_task.delay(item_array, store)
    listing_more_info_sync_task.delay(change_item, store)
    logStatusEnd('Listing Sync End:%s' % store.get('name', ''))

